@@ -138,7 +138,7 @@ __ct_socket_make(ct_socket_t *sock, int op,
 			sock->fd = fd;
 			return fd;
 		}
-		ct_debug("connect() failed: %m");
+		/* no error message - reader does not exist. */
 		break;
 	default:
 		errno = EINVAL;
@@ -322,7 +322,7 @@ int
 ct_socket_getpeername(ct_socket_t *sock, char *buf, size_t len)
 {
 	struct sockaddr_storage ss;
-	socklen_t	slen;
+	socklen_t	slen = sizeof(struct sockaddr_storage);
 
 	getpeername(sock->fd, (struct sockaddr *) &ss, &slen);
 	switch (ss.ss_family) {
