@@ -95,7 +95,7 @@ dir(struct CardTerminal *ct,
 	if (size!=(size_t*)0)
 		*size=0;
 	for (entry=&ct->cwd->dir[0]; *entry; ++entry) {
-		char r[5];
+		unsigned char r[5];
 		int rc;
 
 		r[0]=((*entry)->id>>8)&0xff;
@@ -119,7 +119,7 @@ static int
 hostcf(struct CardTerminal *ct,
 	ct_buf_t *buf, off_t start, size_t length, size_t *size)
 {
-	char data[2];
+	unsigned char data[2];
 	const char *version="OpenCT";
 	int rc;
 
@@ -128,7 +128,7 @@ hostcf(struct CardTerminal *ct,
 	data[1]=strlen(version);
 	if ((rc=put(buf,&start,&length,size,data,2))<0)
 		return rc;
-	if ((rc=put(buf,&start,&length,size,version,strlen(version)))<0)
+	if ((rc=put(buf,&start,&length,size,(const unsigned char *)version,strlen(version)))<0)
 		return rc;
 	return 0;
 }
@@ -168,7 +168,7 @@ CardTerminalFile_select(struct CardTerminal *ct,
 	int id, ct_buf_t *buf)
 {
 	struct CardTerminalFile *cur=(struct CardTerminalFile*)0;
-	char r[12]={ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x00 };
+	unsigned char r[12]={ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x00 };
 	size_t size=0;
 
 	if (id==0x3f00)
@@ -300,6 +300,67 @@ bad_length:
 	return ctapi_error(rbuf, CTBCS_SW_BAD_LENGTH);
 }
 
+static int
+ctapi_set_interface_parameter(struct CardTerminal *ct, char p1, char p2,
+                ct_buf_t *sbuf, ct_buf_t *rbuf)
+{
+	unsigned char 	proto = 0xff;
+	unsigned int slot;
+
+	if ((p1 == 0x00) || (p1 > 2))
+		return ctapi_error(rbuf, CTBCS_SW_BAD_PARAMS);
+	slot = p1 - 1;
+	if (p2 != 0x00)
+                return ctapi_error(rbuf, CTBCS_SW_BAD_PARAMS);
+	while (ct_buf_avail(sbuf)) {
+		unsigned char   type, len, val;
+
+                if (ct_buf_get(sbuf, &type, 1) < 0
+                 || ct_buf_get(sbuf, &len, 1) < 0
+                 || ct_buf_avail(sbuf) < len)
+                        goto bad_length;
+                                                                                                                             
+                if (type == CTBCS_TAG_TPP) { /* Transmission Protocol Preference */
+			if (len != 1)
+				goto bad_length;
+                        ct_buf_get(sbuf, &proto, len);
+			switch (proto) {
+			case 0x01:	proto = IFD_PROTOCOL_T0; /* T=0 */
+					break;
+			case 0x02:	proto = IFD_PROTOCOL_T1; /* T=1 */
+					break;
+			case 0x80:
+			case 0x81:
+			case 0x82:
+			case 0x83:	return ctapi_error(rbuf, CTBCS_SW_NOT_SUPPORTED);
+			default:	
+				return ctapi_error(rbuf, CTBCS_SW_BAD_PARAMS);
+			} 
+                } else if (type == CTBCS_TAG_TPC) {	/* Transmission Protocol Conventions */
+                        if (len != 1)
+                                goto bad_length;
+                        ct_buf_get(sbuf, &val, len);
+			switch (val) {
+			case 0x00:
+			case 0x01:	ctapi_error(rbuf, CTBCS_SW_NOT_SUPPORTED);
+			default:	return	ctapi_error(rbuf, CTBCS_SW_BAD_PARAMS);
+			}
+                } else {
+                        /* Ignore unknown tag */
+                        ct_buf_get(sbuf, NULL, len);
+			return  ctapi_error(rbuf, CTBCS_SW_INVALID_TLV);
+                }
+        }
+	if (proto == 0xff)
+		return ctapi_error(rbuf, CTBCS_SW_BAD_PARAMS);
+	if (!ct_card_set_protocol(ct->h, slot, proto)) 
+		return ctapi_error(rbuf, CTBCS_SW_SUCCESS);
+	else 
+		return ctapi_error(rbuf, CTBCS_SW_NOT_EXECUTABLE);
+bad_length:
+	return ctapi_error(rbuf, CTBCS_SW_BAD_LENGTH);	
+}
+
 static int
 ctapi_status(ct_handle *h, ct_buf_t *rbuf)
 {
@@ -344,6 +405,9 @@ ctapi_control(struct CardTerminal *ct,
 	if (rsp_len < 2)
 		return ERR_INVALID;
 
+	if (cmd_len < 4) 
+		return ctapi_error(&rbuf, CTBCS_SW_BAD_LENGTH);
+
 	ct_buf_set(&sbuf, (void *) cmd, cmd_len);
 	ct_buf_init(&rbuf, rsp, rsp_len);
 
@@ -365,8 +429,18 @@ ctapi_control(struct CardTerminal *ct,
         if (le == 0) le = 256;
 
 	switch ((cmd[0]<<8)|cmd[1]) {
-		case (CTBCS_CLA<<8)|CTBCS_INS_RESET: /* compatibility reset command */
-		case (CTBCS_CLA<<8)|0x10:            /* native reset command */
+		case (CTBCS_CLA<<8)|0x10: /* B1 compatibility reset command */
+			if (cmd_len != 5)
+				return  ctapi_error(&rbuf,CTBCS_SW_BAD_LENGTH);
+			if ((cmd_len == 5) && (cmd[4] != 0x00))
+				return  ctapi_error(&rbuf,CTBCS_SW_BAD_LE);
+			rc = ctapi_reset(ct, cmd[2], cmd[3], &rbuf, 0, NULL);
+			break;
+		case (CTBCS_CLA<<8)|CTBCS_INS_RESET: /* RESET_CT command */
+			if (cmd_len > 5) 
+				return  ctapi_error(&rbuf,CTBCS_SW_BAD_LENGTH);
+			if ((cmd_len == 5) && (cmd[4] != 0x00))
+				return  ctapi_error(&rbuf,CTBCS_SW_BAD_LE);
 			rc = ctapi_reset(ct, cmd[2], cmd[3], &rbuf, 0, NULL);
 			break;
 		case (CTBCS_CLA<<8)|CTBCS_INS_REQUEST_ICC:
@@ -385,6 +459,9 @@ ctapi_control(struct CardTerminal *ct,
 			} else
 				rc = CardTerminalFile_select(ct, (id[0]<<8)|id[1],&rbuf);
 			break;
+		case (CTBCS_CLA_2<<8)|CTBCS_INS_SET_INTERFACE_PARAM:
+			rc = ctapi_set_interface_parameter(ct, cmd[2], cmd[3], &sbuf, &rbuf);
+			break;
 		default:
 			if (cmd[0]!=CTBCS_CLA && cmd[0]!=0x00) {
 				ct_error("Bad CTBCS APDU, cla=0x%02x", cmd[0]);
@@ -410,7 +487,7 @@ ctapi_control(struct CardTerminal *ct,
  */
 static int
 ctapi_transact(struct CardTerminal *ct, int nslot,
-		const char *cmd, size_t cmd_len,
+		const unsigned char *cmd, size_t cmd_len,
 		void *rsp, size_t rsp_len)
 {
 	static const unsigned char select_kvk[11]={ 0x00, 0xa4, 0x04, 0x00, 0x06, 0xd2, 0x80, 0x00, 0x00, 0x01, 0x01 };
@@ -479,7 +556,8 @@ CT_init(unsigned short ctn, unsigned short pn)
 	ct_lock_handle lock;
 	int i;
 
-	if ((ct=malloc(sizeof(struct CardTerminal)))==(struct CardTerminal*)0)
+	ct=(struct CardTerminal*) malloc(sizeof(struct CardTerminal));
+	if (ct==NULL)
 		return ERR_MEMORY;
 	if (!(h = ct_reader_connect(pn))) {
 		free(ct);
