@@ -303,10 +303,11 @@ static int
 kaan_card_status(ifd_reader_t *reader, int slot, int *status)
 {
 	kaan_status_t	*st = (kaan_status_t *) reader->driver_data;
-	unsigned char	buffer[16] = { 0x20, 0x13, slot+1, 0x80, 0x00 };
+	unsigned char	buffer[16] = { 0x20, 0x13, 0x00, 0x80, 0x00 };
 	unsigned char	*byte;
 	int		rc, n;
 
+	buffer[2] = slot+1;
 	ifd_debug(1, "slot=%d", slot);
 	if (!st->frozen
 	 && st->last_activity + FREEZE_DELAY < time(NULL)
@@ -416,7 +417,7 @@ kaan_do_reset(ifd_reader_t *reader, int nslot,
 		/* size. */
 		memcpy(atr, buffer, got);
 		if (got == 4 && st->reader_type == TYPE_B1)
-			kaan_set_protocol(reader, nslot, IFD_PROTOCOL_4433);
+			kaan_set_protocol(reader, nslot, IFD_PROTOCOL_EUROCHIP);
 		if ((rc = kaan_sync_detect(reader, nslot)) < 0)
 			return rc;
 		break;
@@ -442,8 +443,9 @@ kaan_do_reset(ifd_reader_t *reader, int nslot,
 static int
 kaan_card_reset(ifd_reader_t *reader, int nslot, void *result, size_t size)
 {
-	unsigned char	cmd[5] = { 0x20, 0x10, nslot+1, 0x01, 0x00 };
+	unsigned char	cmd[5] = { 0x20, 0x10, 0x00, 0x01, 0x00 };
 
+	cmd[2] = nslot + 1;
 	ifd_debug(1, "called.");
 	return kaan_do_reset(reader, nslot, cmd, 5, (unsigned char *) result, size, 0);
 }
@@ -457,9 +459,10 @@ kaan_card_request(ifd_reader_t *reader, int slot,
 			void *atr, size_t atr_len)
 {
 	ct_buf_t	buf;
-	unsigned char	buffer[256] = { 0x20, 0x17, slot+1, 0x01, 0x00 };
+	unsigned char	buffer[256] = { 0x20, 0x17, 0x00, 0x01, 0x00 };
 	int		n;
 
+	buffer[2] = slot + 1;
 	/* Build the APDU, which is basically a modified CTBCS OUTPUT command */
 	ct_buf_init(&buf, buffer, sizeof(buffer)-1);
 	ctbcs_begin(&buf, 0x17, slot + 1, 0x01);
@@ -482,12 +485,13 @@ static int
 kaan_set_protocol(ifd_reader_t *reader, int nslot, int proto)
 {
 	kaan_status_t	*st = (kaan_status_t *) reader->driver_data;
-	unsigned char	cmd[] = { 0x80, 0x60, nslot+1, 0x00, 0x03, 0x22, 0x01, 0x00 };
+	unsigned char	cmd[] = { 0x80, 0x60, 0x00, 0x00, 0x03, 0x22, 0x01, 0x00 };
 	unsigned char	buffer[2];
 	unsigned short	sw;
 	ifd_slot_t	*slot;
 	int		rc;
 
+	cmd[2] = nslot + 1;
 	ifd_debug(1, "proto=%d", proto);
 
 	switch (proto) {
@@ -497,10 +501,7 @@ kaan_set_protocol(ifd_reader_t *reader, int nslot, int proto)
 	case IFD_PROTOCOL_I2C_LONG:  cmd[7] = 0x80; break;
 	case IFD_PROTOCOL_3WIRE:     cmd[7] = 0x81; break;
 	case IFD_PROTOCOL_2WIRE:     cmd[7] = 0x82; break;
-	case IFD_PROTOCOL_4401:      cmd[7] = 0x90; break;
-	case IFD_PROTOCOL_4402:      cmd[7] = 0x91; break;
-	case IFD_PROTOCOL_4403:      cmd[7] = 0x92; break;
-	case IFD_PROTOCOL_4433:      cmd[7] = 0x93; break;
+	case IFD_PROTOCOL_EUROCHIP:  cmd[7] = 0x93; break;
 	default:
 		ifd_debug(1, "kaan_set_protocol: protocol %d not supported", proto);
 		return -1;
@@ -511,7 +512,7 @@ kaan_set_protocol(ifd_reader_t *reader, int nslot, int proto)
 		return rc;
 	/* B1 returns 6985 for German KVK health care cards */
 	if (sw != 0x9000 && sw != 0x6985) {
-		ifd_debug(1, "kaan_set_protocol: protocol %d not supported, sw=04%x", proto);
+		ifd_debug(1, "kaan_set_protocol: protocol %d not supported, sw=%04hx", proto, sw);
 		return -1;
 	}
 
@@ -721,16 +722,10 @@ kaan_sync_detect(ifd_reader_t *reader, int nslot)
 		protocol = IFD_PROTOCOL_2WIRE;
 		break;
 	case 0x90:
-		protocol = IFD_PROTOCOL_4401;
-		break;
 	case 0x91:
-		protocol = IFD_PROTOCOL_4402;
-		break;
 	case 0x92:
-		protocol = IFD_PROTOCOL_4403;
-		break;
 	case 0x93:
-		protocol = IFD_PROTOCOL_4433;
+		protocol = IFD_PROTOCOL_EUROCHIP;
 		break;
 	default:
 		ct_error("kaan_sync_detect: unknown card protocol 0x%x", protocol);
