@@ -39,6 +39,7 @@ static struct cmd_name {
 	{ CT_CMD_MEMORY_WRITE, "CT_CMD_MEMORY_WRITE" },
 	{ CT_CMD_TRANSACT_OLD, "CT_CMD_TRANSACT_OLD" },
 	{ CT_CMD_TRANSACT, "CT_CMD_TRANSACT" },
+	{ CT_CMD_SET_PROTOCOL, "CT_CMD_SET_PROTOCOL" },
 	{ 0, NULL },
 };
 
@@ -76,6 +77,8 @@ static int do_memory_write(ifd_reader_t *, int,
 			ct_tlv_parser_t *, ct_tlv_builder_t *);
 static int do_transact_old(ifd_reader_t *, int,
 			ct_buf_t *, ct_buf_t *);
+static int do_set_protocol(ifd_reader_t *, int,
+			ct_tlv_parser_t *, ct_tlv_builder_t *);
 
 int
 ifdhandler_process(ct_socket_t *sock, ifd_reader_t *reader,
@@ -153,7 +156,9 @@ ifdhandler_process(ct_socket_t *sock, ifd_reader_t *reader,
 	case CT_CMD_TRANSACT:
 		rc = do_transact(reader, unit, &args, &resp);
 		break;
-
+	case CT_CMD_SET_PROTOCOL:
+                rc = do_set_protocol(reader, unit, &args, &resp);
+                break;
 	default:
 		return IFD_ERROR_INVALID_CMD;
 	}
@@ -407,6 +412,26 @@ do_transact_old(ifd_reader_t *reader, int unit, ct_buf_t *args, ct_buf_t *resp)
 	return 0;
 }
 
+int
+do_set_protocol(ifd_reader_t *reader, int unit,
+                ct_tlv_parser_t *args, ct_tlv_builder_t *resp)
+{
+        unsigned int    protocol = 0xFF;
+        int             rc;
+                                                                                                                             
+        if (unit > reader->nslots)
+                return IFD_ERROR_INVALID_SLOT;
+                                                                                                                             
+        ct_tlv_get_int(args, CT_TAG_PROTOCOL, &protocol);
+                                                                                                                             
+        rc = ifd_set_protocol(reader, unit, protocol);
+        if (rc < 0)
+                return rc;
+                                                                                                                             
+        return 0;
+}
+
+
 /*
  * Synchronous ICC read/write
  */
