@@ -8,7 +8,7 @@
  */
 
 #include "internal.h"
-#if /* !defined (__NetBSD__) && !defined(__FreeBSD__) && !defined(__OpenBSD__) && */ !defined(__linux__)
+#if !defined(sun) && !defined (__NetBSD__) && !defined(__FreeBSD__) && !defined(__OpenBSD__) && !defined(__linux__)
 #include <sys/types.h>
 #include <stdio.h>
 #include <openct/driver.h>
@@ -20,8 +20,22 @@ ifd_sysdep_device_type(const char *name)
 }
 
 /*
- * USB control command
+ * USB handling
  */
+int
+ifd_sysdep_usb_poll_presence(ifd_device_t *dev, struct pollfd *pfd)
+{
+#if 0
+	if (pfd->revents & POLLHUP)
+		return 0;
+	pfd->fd = dev->fd;
+	pfd->events = POLLHUP;
+	return 1;
+#else
+	return -1;
+#endif
+}
+
 int
 ifd_sysdep_usb_control(ifd_device_t *dev,
 		unsigned int requesttype,
@@ -33,6 +47,40 @@ ifd_sysdep_usb_control(ifd_device_t *dev,
 	return -1;
 }
 
+int
+ifd_sysdep_usb_set_configuration(ifd_device_t *dev, int config) 
+{
+     return -1;
+}
+
+int
+ifd_sysdep_usb_set_interface(ifd_device_t *dev, int ifc, int alt) 
+{
+     return -1;
+}
+
+int
+ifd_sysdep_usb_claim_interface(ifd_device_t *dev, int interface) 
+{
+     return -1;
+}
+
+int
+ifd_sysdep_usb_release_interface(ifd_device_t *dev, int interface) 
+{
+     return -1;
+}
+
+/*
+ * USB bulk transfer
+ */
+int
+ifd_sysdep_usb_bulk(ifd_device_t *dev, int ep, void *buffer, size_t len,
+		    long timeout) 
+{
+     return -1;
+}
+
 /*
  * USB URB capture
  */
