@@ -90,6 +90,8 @@ enum {
 	IFD_PROTOCOL_T1_CHECKSUM_LRC,
 	IFD_PROTOCOL_T1_IFSC,
 	IFD_PROTOCOL_T1_IFSD,
+	IFD_PROTOCOL_T1_STATE,
+	IFD_PROTOCOL_T1_MORE,
 };
 
 enum {
